from pandas.core.reshape.concat import concat as concat
from pandas.core.reshape.encoding import (
    from_dummies as from_dummies,
    get_dummies as get_dummies,
)
from pandas.core.reshape.melt import (
    lreshape as lreshape,
    melt as melt,
    wide_to_long as wide_to_long,
)
from pandas.core.reshape.merge import (
    merge as merge,
    merge_asof as merge_asof,
    merge_ordered as merge_ordered,
)
from pandas.core.reshape.pivot import (
    crosstab as crosstab,
    pivot as pivot,
    pivot_table as pivot_table,
)
from pandas.core.reshape.tile import (
    cut as cut,
    qcut as qcut,
)
