from pandas._typing import (
    Dtype,
    Scalar,
)

from pandas.core.dtypes.base import ExtensionDtype
from pandas.core.dtypes.dtypes import (
    register_extension_dtype as register_extension_dtype,
)

# merged types from pylance

class SparseDtype(ExtensionDtype):
    def __init__(self, dtype: Dtype = ..., fill_value: Scalar | None = ...) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other) -> bool: ...
    @property
    def fill_value(self): ...
    @property
    def kind(self): ...
    @property
    def type(self): ...
    @property
    def subtype(self): ...
    @property
    def name(self): ...
    @classmethod
    def construct_array_type(cls): ...
    @classmethod
    def construct_from_string(cls, string): ...
    @classmethod
    def is_dtype(cls, dtype): ...
    def update_dtype(self, dtype): ...
